// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;


import javax.swing.event.ChangeListener;

import org.freehep.util.Value;

/**
 * Condition that filter displayable objects based on a single attribute value.
 * 
 * @author Mark Donszelmann
 * @author Dmitry Onoprienko
 * @version $Id: Condition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public interface Condition {
  
// -- Applying condition : -----------------------------------------------------

  /** Returns <tt>true</tt> is the value satisfies this condition. */
  boolean pass(Value value);
  
// -- Getters : ----------------------------------------------------------------

  /** Returns the Cut this Condition belongs to. */
  Cut getCut();

  /** Returns <tt>true</tt> is this condition is enabled. */
  boolean isEnabled();

  /** Returns <tt>true</tt> is this condition is inverted. */
  boolean isInverted();

  /** Returns <tt>ConditionDefinition</tt> associated with this condition. */
  ConditionDefinition getDefinition();

  /** Returns status string describing what attribute values satisfy this condition in its current state. */
  String getStatus();

  /**
   * Returns criteria used by this condition.
   * The actual type of the return value depends on the type of the condition.
   */
  Object get();

// -- Setters : ----------------------------------------------------------------

  /** Sets the Cut this Condition belongs to. */
  void setCut(Cut cut);

  /** Enables or disables this condition. */
  void setEnabled(boolean state);

  /** Sets inversion state of this condition. */
  void setInverted(boolean state);

  /**
   * Sets criteria to be used by this condition.
   * The actual type of the argument depends on the type of the condition.
   */
  void set(Object value);

  /**
   * Resets condition to its default state.
   * For most kinds of conditions, that means nothing (everything) is cut if the condition
   * is not (is) inverted. Boolean conditions are an exception.
   */
  void reset();

}
