// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import org.freehep.util.Value;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: DoubleConditionDefinition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class DoubleConditionDefinition extends AbstractConditionDefinition {

// -- Private parts : ----------------------------------------------------------

  private double[] _range;

// -- Construction and initialization : ----------------------------------------

  public DoubleConditionDefinition(String name) {
    super(name);
  }

  public DoubleConditionDefinition(String name, String unit, String description) {
    super(name, unit, description);
  }

  
// -- Type : -------------------------------------------------------------------

  /** Returns the type of this condition. */
  public Type getType() {
    return Type.DOUBLE;
  }


// -- Creating Conditions : ----------------------------------------------------

  /** Returns a new {@link Condition} object linked to this ConditionDefinition. */
  public DoubleCondition createCondition(boolean enabled, boolean inverted) {
    double[] range = getDomain();
    DoubleCondition c = new DoubleCondition(this, enabled, inverted, range[0], range[1]);
    _conditions.add(c);
    return c;
  }


// -- Domain : -----------------------------------------------------------------

  /**
   * Sets the domain of possible values for the condition.
   * The argument passed to this method should be a two-element int array: [min,max].
   */
  public void setDomain(Object domain) {
    _range = (double[])domain;
    fireStateChanged();
  }

  /**
   * Sets the range of possible values for the condition.
   */
  public void setDomain(double min, double max) {
    _range = new double[] {min,max};
    fireStateChanged();
  }

  /**
   * Returns the domain of possible values for the condition.
   * The actual return type is a two-element <tt>double</tt> array: [min,max].
   */
  public double[] getDomain() {
    return _range == null ? new double[] {0.,0.} : _range;
  }

  /** Updates the domain to ensure it contains the specified value. */
  public void addToDomain(Value value) {
    addToDomain(value.getDouble());
  }

  /** Updates the domain to ensure it contains the specified value. */
  public void addToDomain(double value) {
    if (_range == null) {
      _range = new double[] {value,value};
    } else if (value < _range[0]) {
      _range[0] = value;
      fireStateChanged();
    } else if (value > _range[1]) {
      _range[1] = value;
      fireStateChanged();
    }
  }

// -----------------------------------------------------------------------------
}
