// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import org.freehep.util.Value;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: LongConditionDefinition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class LongConditionDefinition extends AbstractConditionDefinition {

// -- Private parts : ----------------------------------------------------------

  long[] _range;

// -- Construction and initialization : ----------------------------------------

  public LongConditionDefinition(String name) {
    super(name);
  }

  public LongConditionDefinition(String name, String unit, String description) {
    super(name, unit, description);
  }


// -- Type : -------------------------------------------------------------------

  /** Returns the type of this condition. */
  public Type getType() {
    return Type.LONG;
  }


// -- Creating Conditions : ----------------------------------------------------

  /** Returns a new {@link Condition} object linked to this ConditionDefinition. */
  public LongCondition createCondition(boolean enabled, boolean inverted) {
    long[] range = getDomain();
    LongCondition c = new LongCondition(this, enabled, inverted, range[0], range[1]);
    _conditions.add(c);
    return c;
  }


// -- Domain : -----------------------------------------------------------------

  /**
   * Sets the domain of possible values for the condition.
   * The argument passed to this method should be a two-element <tt>int</tt> array: [min,max].
   */
  public void setDomain(Object domain) {
    _range = (long[])domain;
    fireStateChanged();
  }

  /**
   * Sets the range of possible values for the condition.
   */
  public void setDomain(long min, long max) {
    _range = new long[] {min,max};
    fireStateChanged();
  }

  /**
   * Returns the domain of possible values for the condition.
   * The actual return type is a two-element <tt>long</tt> array: [min,max].
   */
  public long[] getDomain() {
    return _range;
  }

  /**
   * Updates the domain to ensure it contains the specified value.
   */
  public void addToDomain(Value value) {
    addToDomain(value.getLong());
  }

  /**
   * Updates the domain to ensure it contains the specified value.
   */
  public void addToDomain(long value) {
    if (_range == null) {
      _range = new long[] {value,value};
    } else if (value < _range[0]) {
      _range[0] = value;
      fireStateChanged();
    } else if (value > _range[1]) {
      _range[1] = value;
      fireStateChanged();
    }
  }

// -----------------------------------------------------------------------------
}
