// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import javax.swing.event.ChangeEvent;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
abstract public class SliderEditor extends ConditionEditor {

// -- Private parts : ----------------------------------------------------------

  Slider _slider;
  SliderPanel _panel;

  protected boolean _applyImmediately; // true if condition should be modified while knobs are being dragged

// -- Construction and initialization : ----------------------------------------

  public SliderEditor(SliderPanel panel) {
    _slider = panel.getSlider();
    _panel = panel;
  }

// -- Setters : ----------------------------------------------------------------

  /** Determines range of values to be accessible by the slider. */
  abstract void setDomain();

  /** Prepares the model for conversions between slider and model coordinates. */
  abstract void setScale();

  public void setCondition(Condition condition) {
    super.setCondition(condition);
    setDomain();
    _slider.setModel(this);
    _panel.setModel(this);
    _applyImmediately = _panel.getApplyWhileChangingBox().isSelected();
  }

// -- Getters : ----------------------------------------------------------------

  public SliderPanel getPanel() {
    return _panel;
  }

  boolean getApplyImmediately() {
    return _applyImmediately;
  }

  abstract boolean getMark() throws IllegalArgumentException;

  /** Returns low bound of the range in slider coordinates. */
  abstract int getLow();

  /** Returns high bound of the range in slider coordinates. */
  abstract int getHigh();

  /** Returns String representation of low bound. */
  abstract String getLowString();

  /** Returns String representation of high bound. */
  abstract String getHighString();


// -- Updates from ConditionDefinition : ---------------------------------------

  public void stateChanged(ChangeEvent event) {
    setDomain();
    _slider.doLayout();
    updateFields();
  }

// -- Updates to/from slider : -------------------------------------------------

  /** Called by Slider when its knobs have been moved. */
  abstract void updateFromSlider(int low, int high, boolean isValueChanging);

  /** Called by Slider when its low knob has been moved. */
  abstract void updateFromSliderLow(int low, boolean valueIsAdjusting);

  /** Called by Slider when its high knob has been moved. */
  abstract void updateFromSliderHigh(int high, boolean valueIsAdjusting);

  /** Updates Slider after a change in Condition range. */
  void updateSlider() {
    if (_slider == null) return;
    int low = getLow();
    int high = getHigh();
    _slider.updateKnobPositions(low, high);
    _slider.updateMarkPositions(low, high);
  }

  /** Updates Slider after the low bound of Condition has changed. */
  void updateSliderLow() {
    if (_slider == null) return;
    int low = getLow();
    _slider.updateKnobPositionLow(low);
    _slider.updateMarkPositionLow(low);
  }

  /** Updates Slider after the high bound of Condition has changed. */
  void updateSliderHigh() {
    if (_slider == null) return;
    int high = getHigh();
    _slider.updateKnobPositionHigh(high);
    _slider.updateMarkPositionHigh(high);
  }


// -- Updates to/from panel : --------------------------------------------------

  void setApplyImmediately(boolean state) {
    _applyImmediately = state;
  }
  
  abstract void setMark(boolean state);

  abstract void reset();

  abstract void updateFromLowField(String low) throws NumberFormatException;

  abstract void updateFromHighField(String high) throws NumberFormatException;

  void updateLowField() {
    if (_panel != null) _panel.getLowField().setText(getLowString());
  }

  void updateHighField() {
    if (_panel != null) _panel.getHighField().setText(getHighString());
  }

  void updateFields() {
    if (_panel != null) {
      _panel.getLowField().setText(getLowString());
      _panel.getHighField().setText(getHighString());
    }
  }

// -- Creating ruller for Slider : ---------------------------------------------

  abstract Slider.Ruler createRuler();

// -----------------------------------------------------------------------------
}
