// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class SliderPanel extends JPanel {

// -- Private parts : ----------------------------------------------------------

  private SliderEditor _model;

  private final int _padding = 3;

  protected JLabel _nameLabel;
  protected Slider _slider;
  protected JTextField _lowField, _highField;
  protected JCheckBox _markBox;
  protected JCheckBox _applyBox;
  protected JButton _resetButton;

//  private Action _logAction;
//  private Action _snapAction;

// -- Construction and initialization : ----------------------------------------

  public SliderPanel() {

    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

    _nameLabel = new JLabel();
    _nameLabel.setAlignmentX(CENTER_ALIGNMENT);
//    _nameLabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
    add(_nameLabel);

    add(Box.createRigidArea(new Dimension(0,_padding)));

    _highField = new JTextField();
    _highField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (_model != null) {
          try {
            _model.updateFromHighField(_highField.getText());
          } catch (NumberFormatException x) {
            _highField.setText(_model.getHighString());
          }
        }
      }
    });
    Box box = Box.createHorizontalBox();
    box.add(Box.createRigidArea(new Dimension(_padding, 0)));
    box.add(_highField);
    box.add(Box.createRigidArea(new Dimension(_padding, 0)));
    add(box);

    add(Box.createRigidArea(new Dimension(0,_padding)));

    _slider = new Slider();
    add(_slider);

    add(Box.createRigidArea(new Dimension(0,_padding)));

    _lowField = new JTextField();
    _lowField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (_model != null) {
          try {
            _model.updateFromLowField(_lowField.getText());
          } catch (NumberFormatException x) {
            _lowField.setText(_model.getLowString());
          }
        }
      }
    });
    box = Box.createHorizontalBox();
    box.add(Box.createRigidArea(new Dimension(_padding, 0)));
    box.add(_lowField);
    box.add(Box.createRigidArea(new Dimension(_padding, 0)));
    add(box);

    add(Box.createRigidArea(new Dimension(0,_padding)));

    _markBox = new JCheckBox("Snap");
    _markBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (_model != null) _model.setMark(_markBox.isSelected());
      }
    });
    _markBox.setAlignmentX(CENTER_ALIGNMENT);
    add(_markBox);

    add(Box.createRigidArea(new Dimension(0,_padding)));

    _applyBox = new JCheckBox("Apply", true);
    _applyBox.setToolTipText("Check to apply changes as you move sliders");
    _applyBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (_model != null) _model.setApplyImmediately(_applyBox.isSelected());
      }
    });
    _applyBox.setAlignmentX(CENTER_ALIGNMENT);
    add(_applyBox);

    add(Box.createRigidArea(new Dimension(0,_padding)));

    _resetButton = new JButton("Reset");
    _resetButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (_model != null) _model.reset();
      }
    });
    _resetButton.setToolTipText("Reset range to include entire domain");
    _resetButton.setAlignmentX(CENTER_ALIGNMENT);
    add(_resetButton);
  }


// -----------------------------------------------------------------------------

  /**
   * Sets Condition to be edited.
   * If called with null argument, resets the editor, eliminating any references
   * to previously edited condition.
   */
  public void setModel(SliderEditor model) {
    _model = null;
    if (model == null) {
      _nameLabel.setText("");
      _nameLabel.setToolTipText(null);
      _highField.setText("");
      _lowField.setText("");
      _slider.setModel(null);
      setEnabled(false);
    } else {
      _nameLabel.setText(model.getCondition().getDefinition().getName());
      _nameLabel.setToolTipText(model.getCondition().getDefinition().getDescription());
      model.updateFields();
      _markBox.setSelected(model.getMark());
      switch (model.getType()) {
        case INTEGER:
        case LONG:
          _markBox.setText("Snap");
          _markBox.setToolTipText("Check to make sliders snap to nearest integral values");
          break;
        case DOUBLE:
          _markBox.setText("Log");
          _markBox.setToolTipText("Check to use log scale");
      }
      setEnabled(true);
      revalidate();
    }
    _model = model;
  }


// -- Access to components : ---------------------------------------------------

  public JLabel getNameLabel() {return _nameLabel;}

  public Slider getSlider() {return _slider;}

  public JTextField getLowField() {return _lowField;}

  public JTextField getHighField() {return _highField;}

  public JCheckBox getMarkBox() {return _markBox;}

  public JCheckBox getApplyWhileChangingBox() {return _applyBox;}

  public JButton getResetButton() {return _resetButton;}


// -- Getters : ----------------------------------------------------------------


// -- Testing code : -----------------------------------------------------------

  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception e) {}
    javax.swing.SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        createAndShowGUI();
      }
    });
  }

  private static void createAndShowGUI() {

    JFrame frame = new JFrame("Range Selector Test");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    JPanel newContentPane = new JPanel();
    newContentPane.setOpaque(true);
    newContentPane.setLayout(new BoxLayout(newContentPane, BoxLayout.Y_AXIS));
    frame.setContentPane(newContentPane);

    doTest(newContentPane);

    frame.setMinimumSize(new Dimension(10, 300));
    frame.setPreferredSize(new Dimension(100, 900));
    frame.pack();
    frame.setVisible(true);
  }

  private static void doTest(final JPanel panel) {

    IntegerConditionDefinition icd = new IntegerConditionDefinition("int condition def");
    icd.setDomain(new int[] {-11,30});
    IntegerCondition ic = new IntegerCondition(icd, -1, 1);

    DoubleConditionDefinition dcd = new DoubleConditionDefinition("double condition def");
    dcd.setDomain(new double[] {0.05,3872365.4});
    DoubleCondition dc = new DoubleCondition(dcd, 1000., 3001.0);

    SliderPanel sliderPanel = new SliderPanel();

//    IntegerSliderEditor intEditor = new IntegerSliderEditor(sliderPanel);
//    intEditor.setCondition(ic);

    DoubleSliderEditor m = new DoubleSliderEditor(sliderPanel);
    m.setCondition(dc);

    panel.add(sliderPanel);
  }

// -----------------------------------------------------------------------------
}
