// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import java.util.*;

import org.freehep.util.Value;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: StringConditionDefinition.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class StringConditionDefinition extends AbstractConditionDefinition {

// -- Private parts : ----------------------------------------------------------

  HashSet<String> _domain;


// -- Construction and initialization : ----------------------------------------

  public StringConditionDefinition(String name) {
    this(name, null, null);
  }

  public StringConditionDefinition(String name, String unit, String description) {
    super(name, unit, description);
    _domain = new HashSet();
  }


// -- Type : -------------------------------------------------------------------

  /** Returns the type of this condition. */
  public Type getType() {
    return Type.STRING;
  }


// -- Creating Conditions : ----------------------------------------------------

  /** Returns a new {@link Condition} object linked to this ConditionDefinition. */
  public StringCondition createCondition(boolean enabled, boolean inverted) {
    StringCondition c = new StringCondition(this, enabled, inverted, _domain);
    _conditions.add(c);
    return c;
  }


// -- Domain : -----------------------------------------------------------------

  /**
   * Sets the domain of possible values for the condition.
   * The argument passed to this method should be a collection of strings.
   */
  public void setDomain(Object domain) {
    _domain = new HashSet((Collection<String>)domain);
    fireStateChanged();
  }

  /**
   * Returns the domain of possible values for the condition.
   * The actual return type is an unmodifiable set of strings.
   */
  public Set<String> getDomain() {
    return Collections.unmodifiableSet(_domain);
  }

  /**
   * Updates the domain to ensure it contains the specified value.
   */
  public void addToDomain(Value value) {
    _domain.add(value.getString());
  }

  /**
   * Updates the domain to ensure it contains the specified value.
   */
  public void addToDomain(String value) {
    _domain.add(value);
  }

// -----------------------------------------------------------------------------
}
