// Copyright 2005-2011, FreeHEP.
package hep.wired.cut;

import java.util.*;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class StringConditionEditor extends ConditionEditor {

// -- Private parts : ----------------------------------------------------------

  private JPanel _panel;

  private ArrayList<String> _data;
  private JTable _table;
  private AbstractTableModel _model;

// -- Construction and initialization : ----------------------------------------

  public StringConditionEditor() {

    _model = new AbstractTableModel() {
      public int getRowCount() {return _data.size();}
      public int getColumnCount() {return 1;}
      public Object getValueAt(int row, int column) {return _data.get(row);}
      public Class<?> getColumnClass(int column) {return String.class;}
    };
    _table = new JTable(_model);
    _table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        if (_condition != null) {
          int[] sel = _table.getSelectedRows();
          ArrayList<String> selection = new ArrayList(sel.length);
          for (int i : sel) {
            selection.add(_data.get(_table.convertRowIndexToModel(i)));
          }
          _condition.set(selection);
        }
      }
    });
    _panel = new JPanel();
    _panel.add(new JScrollPane(_table));
  }

// -----------------------------------------------------------------------------

  /**
   * Sets Condition to be edited.
   * If called with null argument, resets the editor, eliminating any references
   * to previously edited condition.
   */
  public void setCondition(Condition condition) {
    if (condition != _condition) {
      if (_condition != null) {
        _condition.getDefinition().removeChangeListener(this);
      }
      _condition = (StringCondition) condition;
      if (_condition != null) {
        stateChanged(null);
         _condition.getDefinition().addChangeListener(this);
      }
    }
  }

  /**
   * Responds to events from {@link ConditionDefinition}.
   * Updates the domain.
   */
  public void stateChanged(ChangeEvent event) {
    Set<String> all = new LinkedHashSet(getCondition().getDefinition().getDomain());
    Set<String> selection = getCondition().get();
    all.addAll(selection);
    _data = new ArrayList(all);
    _model.fireTableDataChanged();
    _table.clearSelection();
    for (int i = 0; i < _data.size(); i++) {
      if (selection.contains(_data.get(i))) {
        int index = _table.convertRowIndexToView(i);
        _table.addRowSelectionInterval(index, index);
      }
    }
  }

// -- Getters : ----------------------------------------------------------------

  /** Returns GUI for this editor. */
  public JPanel getPanel() {
    return _panel;
  }

  /** Returns the type of condition handled by this editor. */
  public ConditionDefinition.Type getType() {
    return ConditionDefinition.Type.STRING;
  }

  /** Returns Condition being edited. */
  public StringCondition getCondition() {
    return (StringCondition) _condition;
  }

// -----------------------------------------------------------------------------
}
