// Copyright 2004, FreeHEP.
package hep.wired.edit;

import java.awt.Shape;

/**
 * Base class for an animated single edit. This class contains a shape to be
 * animated in a certain number of frames, and handles the animation.
 * 
 * @author Mark Donszelmann
 * @version $Id: AnimatedWiredEdit.java 8598 2006-08-14 20:38:02Z duns $
 */
public abstract class AnimatedWiredEdit extends WiredEdit {

    private Shape initialShape;

    private Shape[] shapes;

    /**
     * Creates an animated edit with zero frames and no shape.
     */
    public AnimatedWiredEdit() {
        this(null, 0);
    }

    /**
     * Creates an animated edit with given initial shape and number of frames.
     */
    public AnimatedWiredEdit(Shape initialShape, int frames) {
        super(frames);
        this.initialShape = initialShape;
        shapes = null;
    }

    /**
     * Returns the initial shape.
     */
    protected Shape getShape() {
        return initialShape;
    }

    /**
     * Returns an array of 'frames' shapes deformed from the given initial
     * shape.
     */
    protected Shape[] getShapes(Shape initialShape, int frames) {
        return null;
    }

    /**
     * Starts the animation by calling getShapes() to get the set of 'framed'
     * shapes. The plot is set into fast mode.
     */
    protected void startAnimation(boolean redo) {
        if (shapes == null)
            shapes = getShapes(initialShape, getFrames());
        getRecordPlot().getGraphicsPanel().setFastMode(true);
    }

    /**
     * Shows frameNo in the plot by drawing its shape.
     */
    protected void showAnimation(int frameNo) {
        getRecordPlot().drawShape(shapes[frameNo]);
    }

    /**
     * Ends the animation by removing the last drawn shape from the plot and
     * changing the plot to normal mode again.
     */
    protected void endAnimation() {
        getRecordPlot().getGraphicsPanel().setFastMode(false);
        getRecordPlot().drawShape(null);
    }

    /**
     * Returns a string representation of the edit.
     */
    public String toString() {
        return super.toString() + "; " + getPresentationName();
    }
}
