// Copyright 2004-2005, FreeHEP.
package hep.wired.edit;

import hep.wired.feature.HasBoundingBox;
import hep.wired.feature.Rotateable3D;
import hep.wired.feature.Translateable;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.awt.Component;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;

import javax.swing.undo.UndoableEdit;

/**
 * Fits plot to window by translation and scaling. If rotation is available a
 * fit (translate and scale) will also be done perpendicular to the visible
 * plane.
 * 
 * @author Mark Donszelmann
 * @version $Id: FitToWindow.java 8598 2006-08-14 20:38:02Z duns $
 */
public class FitToWindow extends WiredCompoundEdit implements GraphicsPanelEdit {

//    private static Logger logger = Logger.getLogger(FitToWindow.class
//            .getPackage().getName());

    /**
     * Creates a Fit to Window edit.
     */
    public FitToWindow() {
        super("FitToWindow");
    }

    public WiredCompoundEdit copy(RecordPlot plot) {
        WiredCompoundEdit copy = new FitToWindow();
        copy.setRecordPlot(plot);

        GraphicsPanel panel = plot.getGraphicsPanel();

        Rectangle2D bbUV = ((HasBoundingBox) panel
                .getFeature(HasBoundingBox.class)).getBoundingBoxForPlot();

        // bail out if no bounding box, or no size on plot, but keep edit.
        if ((bbUV == null) || (plot.getWidth() == 0) || (plot.getHeight() == 0)) {
            copy.addEdit(new NoOperation().copy(plot));
            copy.end();
            return copy;
        }
        double du = bbUV.getX() + bbUV.getWidth() / 2 - plot.getWidth() / 2;
        double dv = bbUV.getY() + bbUV.getHeight() / 2 - plot.getHeight() / 2;
        double dz = 0;
        // System.out.println(new Rectangle((int)bbUV.getX(), (int)bbUV.getY(),
        // (int)bbUV.getWidth(), (int)bbUV.getHeight())+" "+du+" "+dv);

        // make a copy, rotate and calculate dz if possible
        if (panel.supports(Rotateable3D.class)) {
            // FIXME: does NOT work correctly, seems to keep old information,
            // see WIRED-225 and WIRED-227
            // RecordPlot wvPlot = plot.copy("tmp-copy", plot.getRecord(),
            // false);
            RecordPlot wvPlot = plot;
            wvPlot.postEdit(new Rotate(Math.PI / 2, 0, 1, 0));
            Rectangle2D bbWV = ((HasBoundingBox) wvPlot.getGraphicsPanel()
                    .getFeature(HasBoundingBox.class)).getBoundingBoxForPlot();
            // use plot here since wvPlot is not visible (width = 0).
            dz = (bbWV.getX() + bbWV.getWidth() / 2 - plot.getWidth() / 2);
            // System.out.println(new Rectangle((int)bbWV.getX(),
            // (int)bbWV.getY(), (int)bbWV.getWidth(), (int)bbWV.getHeight())+"
            // "+dz);
            wvPlot.postEdit(new Rotate(-Math.PI / 2, 0, 1, 0));
        }

        // center
        Translateable translateable = (Translateable) plot.getGraphicsPanel()
                .getFeature(Translateable.class);
        double t[] = translateable.getModelTranslation(new double[] { -du, dv,
                -dz }, plot.getGraphicsPanel().getViewPort());
        // System.out.println("Plot: "+plot.getWidth()+" "+plot.getHeight());
        // System.out.println(t[0]+" "+t[1]+" "+t[2]/*+tz[0]+" "+tz[1]*/);
        Translate translate = new Translate(t[0], t[1], t[2], bbUV, 5);
        Shape shape = translate.createTransformedShape((Component) plot, bbUV);
        copy.addEdit(translate.copy(plot));

        // zoom
        double sx = plot.getWidth() / (bbUV.getWidth() + 1);
        double sy = plot.getHeight() / (bbUV.getHeight() + 1);
        double s = Math.min(sx, sy);
        copy.addEdit(new Scale(s, s, s, shape, 5).copy(plot));
        copy.end();
        return copy;
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        Translate t = new Translate();
        Scale s = new Scale();
        return t.isSupportedBy(p) && s.isSupportedBy(p);
    }

    public boolean addEdit(UndoableEdit edit) {
        if (edit instanceof FitToWindow) {
            // absorb
            return true;
        }
        return super.addEdit(edit);
    }
}
