// Copyright 2005, FreeHEP.
package hep.wired.edit;

import hep.wired.feature.Scaleable2D;
import hep.wired.feature.Scaleable3D;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;
import hep.wired.util.XYZindices;

import java.util.logging.Logger;

import javax.swing.undo.UndoableEdit;

/**
 * Set Scale of a plot in 2D or 3D.
 * 
 * @author Mark Donszelmann
 * @version $Id: SetScale.java 8598 2006-08-14 20:38:02Z duns $
 */

public class SetScale extends WiredEdit implements GraphicsPanelEdit,
        XYZindices {

    private static Logger logger = Logger.getLogger(SetScale.class.getPackage()
            .getName());

    private double sx, sy, sz;

    private double[] oldS;

    public SetScale(double sx, double sy, double sz) {
        super(0);
        this.sx = sx;
        this.sy = sy;
        this.sz = sz;
        oldS = null;
    }

    public WiredEdit copy(RecordPlot plot) {
        WiredEdit copy = new SetScale(sx, sy, sz);
        copy.setRecordPlot(plot);
        return copy;
    }

    public String getPresentationName() {
        StringBuffer s = new StringBuffer();
        s.append(toString());
        s.append(" (");
        s.append(format.format(sx));
        if ((sy != 1) || (sz != 1)) {
            s.append(", ");
            s.append(format.format(sy));
        }
        if (sz != 1) {
            s.append(", ");
            s.append(format.format(sz));
        }
        s.append(")");
        return s.toString();
    }

    public String toString() {
        return "SetScale";
    }

    public boolean addEdit(UndoableEdit edit) {
        return false;
    }

    public boolean isSupportedBy(GraphicsPanel p) {
        return p.supports(Scaleable2D.class) || p.supports(Scaleable3D.class);
    }

    protected void redoEdit() {
        GraphicsPanel panel = getRecordPlot().getGraphicsPanel();
        Scaleable3D scaleable3D = (Scaleable3D) panel
                .getFeature(Scaleable3D.class);
        if (scaleable3D != null) {
            oldS = scaleable3D.getScale();
            scaleable3D.setScale(sx, sy, sz);
        } else {
            Scaleable2D scaleable2D = (Scaleable2D) panel
                    .getFeature(Scaleable2D.class);
            if (scaleable2D == null) {
                System.err.println("Could not redo: " + this);
                return;
            }
            oldS = scaleable2D.getScale();
            scaleable2D.setScale(sx, sy);
        }
        getRecordPlot().repaint();
        logger.finer(toString());
    }

    protected void undoEdit() {
        if (oldS == null) {
            System.err.println("Could not undo: " + this);
            return;
        }

        GraphicsPanel panel = getRecordPlot().getGraphicsPanel();
        Scaleable3D scaleable3D = (Scaleable3D) panel
                .getFeature(Scaleable3D.class);
        if (scaleable3D != null) {
            scaleable3D.setScale(oldS[X], oldS[Y], oldS[Z]);
        } else {
            Scaleable2D scaleable2D = (Scaleable2D) panel
                    .getFeature(Scaleable2D.class);
            if (scaleable2D == null) {
                System.err.println("Could not undo: " + this);
                return;
            }
            scaleable2D.setScale(oldS[X], oldS[Y]);
        }
        getRecordPlot().repaint();
        logger.finer(toString());
    }
}
