// Copyright 2004, FreeHEP.
package hep.wired.image;

import java.awt.Cursor;
import java.awt.Image;

import javax.swing.Icon;

import org.freehep.util.images.ImageHandler;

/**
 * Facility class to load Images, Icons and Cursors.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredBaseImage.java 8598 2006-08-14 20:38:02Z duns $
 */
public class WiredBaseImage extends ImageHandler {

    // only static methods
    private WiredBaseImage() {
    }

    /**
     * Returns an image for given name, relative to this class.
     */
    public static Image getImage(String name) {
        return getImage(name, WiredBaseImage.class);
    }

    /**
     * Returns the best cursor for given name, relative to this class. The
     * cursor is created with given width and height.
     */
    public static Cursor getBestCursor(String name, int width, int height) {
        return getBestCursor(name, width, height, 0, 0);
    }

    /**
     * Returns the best cursor for given name, relative to this class. The
     * cursor is created with given width and height and hotspot x, y.
     */
    public static Cursor getBestCursor(String name, int width, int height,
            int x, int y) {
        return getBestCursor(name, WiredBaseImage.class, width, height, x, y);
    }

    /**
     * Returns an icon for given name, relative to this class.
     */
    public static Icon getIcon(String name) {
        return getIcon(name, WiredBaseImage.class);
    }

    /**
     * Returns an icon for given name, relative to this class.
     */
    public static Icon getIcon(String name, int size) {
        name = name.replaceAll("%w", Integer.toString(size));
        return getIcon(name);
    }

}
