// Copyright 2004-2006, FreeHEP.
package hep.wired.interaction;

import hep.wired.edit.Translate;
import hep.wired.feature.Translateable;
import hep.wired.image.WiredBaseImage;
import hep.wired.services.GraphicsPanel;
import hep.wired.services.RecordPlot;

import java.awt.Shape;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

import javax.swing.Icon;

import org.freehep.application.Application;

/**
 * Drag/click mouse to translate plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: DragToTranslate.java 8598 2006-08-14 20:38:02Z duns $
 */
public class DragToTranslate extends DefaultInteractionHandler {

    private int xp, yp;

    private final static int cursorSize = 32;

    /**
     * Create a drag handler to generate translate edits.
     */
    public DragToTranslate() {
        super("Drag to Translate");
    }

    public Icon getIcon(int size) {
        return WiredBaseImage.getIcon("Translate%w", size);
    }

    public String getDescription() {
        return "Click to center point, Shift-Click to move center to point, Drag to instant translate.";
    }

    public boolean isSupportedBy(GraphicsPanel panel) {
        return new Translate().isSupportedBy(panel);
    }

    public void reset(RecordPlot plot, InputEvent event) {
        Application.getApplication().setStatusMessage(getDescription());
    }

    public void changeCursor(RecordPlot plot, InputEvent event) {
        plot.setCursor(WiredBaseImage.getBestCursor("TranslateCursor%w",
                cursorSize, cursorSize));
    }

    public void mouseEntered(RecordPlot plot, MouseEvent event) {
        plot.requestFocusInWindow();
        changeCursor(plot, event);
    }

    public void mouseButton1DragStarted(RecordPlot plot, MouseEvent event) {
        xp = event.getX();
        yp = event.getY();
        plot.beginUpdate();
        plot.getGraphicsPanel().setFastMode(true);
    }

    public void mouseButton1DragEnded(RecordPlot plot, MouseEvent event) {
        plot.getGraphicsPanel().setFastMode(false);
        plot.endUpdate();
        plot.repaint();
    }

    public void mouseButton1Dragged(RecordPlot plot, MouseEvent event) {
        int xd = event.getX();
        int yd = event.getY();
        Translateable translateable = (Translateable) plot.getGraphicsPanel()
                .getFeature(Translateable.class);
        double t[] = translateable.getModelTranslation(new double[] { xd - xp,
                -yd + yp, 0 }, plot.getGraphicsPanel().getViewPort());
        plot.postEdit(new Translate(t[0], t[1], t[2]));
        xp = xd;
        yp = yd;
    }

    /**
     * Centers the point you clicked on.
     */
    public void mouseButton1Clicked(RecordPlot plot, MouseEvent event) {
        plot.beginUpdate();
        int w2 = plot.getWidth() / 2;
        int h2 = plot.getHeight() / 2;
        int dx = event.getX() - w2;
        int dy = event.getY() - h2;
        Translateable translateable = (Translateable) plot.getGraphicsPanel()
                .getFeature(Translateable.class);
        if (event.isShiftDown()) {
            // center to click
            Shape shape = new Rectangle2D.Double(w2 - cursorSize, h2
                    - cursorSize, cursorSize * 2, cursorSize * 2);
            double t[] = translateable.getModelTranslation(new double[] { dx,
                    -dy, 0 }, plot.getGraphicsPanel().getViewPort());
            plot.postEdit(new Translate(t[0], t[1], t[2], shape, 5));
        } else {
            // click to center
            Shape shape = new Rectangle2D.Double(event.getX() - cursorSize,
                    event.getY() - cursorSize, cursorSize * 2, cursorSize * 2);
            double t[] = translateable.getModelTranslation(new double[] { -dx,
                    dy, 0 }, plot.getGraphicsPanel().getViewPort());
            plot.postEdit(new Translate(t[0], t[1], t[2], shape, 5));
        }
        plot.endUpdate();
    }

    public String toString() {
        return "Drag to Translate";
    }
}
