// Copyright 2004, FreeHEP.
package hep.wired.plugin;

import hep.wired.services.RecordPlot;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Point;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.UndoableEditSupport;

import org.freehep.application.Application;
import org.freehep.application.mdi.ManagedPage;
import org.freehep.application.mdi.PageContext;
import org.freehep.application.studio.Studio;
import org.freehep.swing.popup.HasPopupItems;
import org.freehep.util.commanddispatcher.CommandProcessor;
import org.freehep.xml.io.XMLIO;
import org.freehep.xml.io.XMLIOManager;
import org.jdom.Element;

/**
 * Defines a page with one plot for inclusion in JAS.
 * 
 * @author Mark Donszelmann
 * @version $Id: WiredPage.java 13824 2011-05-04 18:30:02Z onoprien $
 */
public class WiredPage extends JPanel implements ManagedPage, XMLIO {

//    private static Logger logger = Logger.getLogger(WiredPage.class
//            .getPackage().getName());

    private String name;

    private RecordPlot plot;

    private CommandProcessor processor;

    private WiredPage() {
        this("NotInitialized", null, null);
    }

    public WiredPage(String name, RecordPlot plot, CommandProcessor processor) {
        this.name = name;
        this.plot = plot;
        this.processor = processor;
        setLayout(new BorderLayout());
        if (plot != null)
            init(plot);
    }

    private void init(RecordPlot plot) {
        add((Component) plot, BorderLayout.CENTER);

        // undo support
        if ((processor != null) && (processor instanceof UndoableEditListener)) {
            UndoableEditSupport undoSupport = plot.getUndoableEditSupport();
            if (undoSupport != null)
                undoSupport
                        .addUndoableEditListener((UndoableEditListener) processor);
        }
    }

    public String getName() {
        return name;
    }

    public RecordPlot getRecordPlot() {
        return plot;
    }

    public void setChanged() {
        if (processor != null)
            processor.setChanged();
    }

    //
    // ManagedPage Interface
    //
    public boolean close() {
        return true;
    }

    public void pageClosed() {
    }

    public void pageIconized() {
    }

    public void pageDeiconized() {
    }

    public void pageSelected() {
        if (processor != null) ((Studio) Application.getApplication()).getCommandTargetManager().add(processor);
        plot.setSelected(true);
    }

    public void pageDeselected() {
        if (processor != null)
            ((Studio) Application.getApplication()).getCommandTargetManager()
                    .remove(processor);
        plot.setSelected(false);
    }

    public void setPageContext(PageContext context) {
    }

    //
    // HasPopupItems interface
    //
    public JPopupMenu modifyPopupMenu(JPopupMenu menu, Component source, Point p) {
        return (plot instanceof HasPopupItems) ? ((HasPopupItems) plot)
                .modifyPopupMenu(menu, source, p) : menu;
    }

    //
    // XMLIO
    //
    public void save(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        nodeEl.setAttribute("name", name);
        nodeEl.addContent(xmlioManager.save(plot));
    }

    public void restore(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        name = nodeEl.getAttributeValue("name");
        Iterator i = nodeEl.getChildren().iterator();
        plot = (RecordPlot) xmlioManager.restore((Element) i.next());
        init(plot);
    }
}
