// Copyright 2003-2005, FreeHEP.
package hep.wired.services;

import java.awt.Cursor;
import java.awt.Shape;

import javax.swing.event.ChangeListener;
import javax.swing.undo.UndoableEditSupport;

import org.freehep.swing.undo.UndoableEditProcessor;

/**
 * Defines a single event display plot.
 * 
 * @author Mark Donszelmann
 * @version $Id: RecordPlot.java 8598 2006-08-14 20:38:02Z duns $
 */
public interface RecordPlot extends UndoableEditProcessor {

    /**
     * Adds a change listener
     */
    public void addChangeListener(ChangeListener listener);

    /**
     * Removbes a change listener
     */
    public void removeChangeListener(ChangeListener listener);

    /**
     * Returns a copy of this plot, attached to the given objects, from the
     * given parameters.
     */
    public RecordPlot copy(String name, Object record, boolean supportUndo);

    /**
     * Returns the undo support object, or null if undo was not switched on.
     */
    public UndoableEditSupport getUndoableEditSupport();

    /**
     * Returns the name of the plot.
     */
    public String getName();

    /**
     * Returns the associated Graphics Panel.
     */
    public GraphicsPanel getGraphicsPanel();

    /**
     * Called when this plot is selected/unselected.
     */
    public void setSelected(boolean selected);

    /**
     * Returns true if this plot (and its projection) supports the given
     * interactionHandler.
     */
    public boolean supports(InteractionHandler interactionHandler);

    /**
     * Sets the interaction handler on the plot.
     */
    public void setInteractionHandler(InteractionHandler interactionHandler);

    /**
     * Return the current interaction handler.
     */
    public InteractionHandler getInteractionHandler();

    /**
     * Sets the plot to use this Object.
     */
    public void setRecord(Object record);

    /**
     * Returns the current Object.
     */
    public Object getRecord();

    /**
     * Draws the shape for feedback/interactivity purposes
     */
    public void drawShape(Shape shape);

    //
    // Some component methods
    /**
     * Returns X of plot.
     */
    public int getX();

    /**
     * Returns Y of plot.
     */
    public int getY();

    /**
     * Returns width of plot.
     */
    public int getWidth();

    /**
     * Returns height of plot.
     */
    public int getHeight();

    /**
     * Sets the cursor of plot
     */
    public void setCursor(Cursor c);

    /**
     * Requests focus in plot
     */
    public boolean requestFocusInWindow();

    /**
     * Repaint the plot
     */
    public void repaint();
}
