package hep.wired.util;

import java.util.*;
import javax.swing.table.AbstractTableModel;

import org.freehep.swing.table.SortableTableModel;

/**
 * Keeps a list sorted in place.
 * 
 * @author Mark Donszelmann
 * @version $Id: SortableListTableModel.java 13931 2012-02-09 00:58:03Z onoprien $
 */
public abstract class SortableListTableModel extends AbstractTableModel implements SortableTableModel {

  private String[] names;
  private Class[] types;
  protected List keys;
  private transient int sortOnColumn = UNSORTED;
  private transient boolean sortAscending = true;

  public SortableListTableModel(String[] names, Class[] types, List keys) {
    this.names = names;
    this.types = types;
    this.keys = keys;
  }

  protected List getKeys() {
    return keys;
  }

  public void setKeys(List keys) {
    this.keys = keys;
    fireTableStructureChanged();
  }

  public String getColumnName(int col) {
    return names[col];
  }

  public int getColumnCount() {
    return names.length;
  }

  public Class getColumnClass(int c) {
    return (c < types.length) ? types[c] : String.class;
  }

  public int getRowCount() {
    return keys.size();
  }

  public int getSortOnColumn() {
    return sortOnColumn;
  }

  public boolean isSortAscending() {
    return sortAscending;
  }

  public void sort(int column, boolean ascending) {
    if ((column != sortOnColumn) || (ascending != sortAscending)) {
      sortOnColumn = column;
      sortAscending = ascending;
      sort();
    }
    super.fireTableDataChanged();
  }

  public void fireTableDataChanged() {
    sort();
    super.fireTableDataChanged();
  }

  public void fireTableRowsInserted(int firstRow, int lastRow) {
    sort();
    super.fireTableRowsInserted(firstRow, lastRow);
  }

  public void fireTableRowsUpdated(int firstRow, int lastRow) {
    sort();
    super.fireTableRowsUpdated(firstRow, lastRow);
  }

  public void fireTableRowsDeleted(int firstRow, int lastRow) {
    sort();
    super.fireTableRowsDeleted(firstRow, lastRow);
  }

  public void fireTableCellUpdated(int row, int column) {
    if (column == sortOnColumn) {
      sort();
    }
    super.fireTableCellUpdated(row, column);
  }

  public void fireTableStructureChanged() {
    sortOnColumn = UNSORTED;
    sortAscending = true;
    sort();
    super.fireTableStructureChanged();
  }

  protected abstract int compareKeys(Object key1, Object key2, int column,
          boolean ascending);

  private void sort() {
    Collections.sort(keys, new Comparator() {

      public int compare(Object o1, Object o2) {
        return compareKeys(o1, o2, sortOnColumn, sortAscending);
      }
    });
  }
}
