// Copyright 2004-2005, FreeHEP.
package hep.wired.util;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.freehep.swing.layout.TableLayout;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: TitledPanel.java 8598 2006-08-14 20:38:02Z duns $
 */
public class TitledPanel extends JPanel {

    private JLabel title;
    private boolean[] state = new boolean[0];
    private boolean mainState;
    private Component main;
    private JScrollPane scrollPane;

    public TitledPanel(String text, Component main) {
        this(text, main, new Component[0]);
    }

    public TitledPanel(String text, Component main, Component extra) {
        this(text, main, new Component[] { extra });
    }

    public TitledPanel(String text, Component main, Component[] extra) {
        super();
        title = new JLabel(text);

        setLayout(new TableLayout());
        add("0 0 [3 6 3 3] w", title);
        for (int i = 0; i < extra.length; i++) {
            add((i + 1) + " 0 [3 3 3 3]", extra[i]);
        }
        if (main != null) {
            this.main = main;
            mainState = main.isEnabled();
            scrollPane = new JScrollPane(main, 
                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            add("0 1 * * [3 3 3 3] wh", scrollPane);
        }
    }

    public void setTitle(String text) {
        title.setText(text);
        revalidate();
        repaint();
    }
    
    public void setVerticalScrollBarPolicy(int policy) {
        scrollPane.setVerticalScrollBarPolicy(policy);
    }
    
    public void setHorizontalScrollBarPolicy(int policy) {
        scrollPane.setHorizontalScrollBarPolicy(policy);
    }
    
    public void setEnabled(boolean enabled) {
        if (isEnabled() == enabled) return;
        
        int n = getComponentCount();
        if (enabled) {
            for (int i=0; i<n; i++) {
                getComponent(i).setEnabled(i<state.length ? state[i] : true);
            }
            main.setEnabled(mainState);
        } else {
            state = new boolean[n];
            for (int i=0; i<n; i++) {
                Component c = getComponent(i);
                state[i] = c.isEnabled();
                c.setEnabled(false);
            }
            mainState = main.isEnabled();
            main.setEnabled(false);
        }
        super.setEnabled(enabled);
    }
}
