package hep.wired.util.tree;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.EventObject;
import javax.swing.AbstractCellEditor;
import javax.swing.JTree;
import javax.swing.tree.TreeCellEditor;

/**
 *
 * @author Dmitry Onoprienko
 * @version $Id: $
 */
public class WiredTreeCellEditor extends AbstractCellEditor implements TreeCellEditor {

// -- Private parts : ----------------------------------------------------------

  protected WiredTree _tree;
  protected WiredTreeCellRenderer _renderer;

// -- Constructors : -----------------------------------------------------------

  public  WiredTreeCellEditor(final WiredTree tree) {
    _tree = tree;
    _renderer = new WiredTreeCellRenderer(tree);
    _renderer._branchBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        _renderer._node.setMarkedBranch(_renderer._branchBox.isSelected());
        tree.getModel().fireMarkChanged();
        fireEditingStopped();
      }
    });
    _renderer._nodeBox.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        _renderer._node.setMarkedNode(_renderer._nodeBox.isSelected());
        tree.getModel().fireMarkChanged();
        fireEditingStopped();
      }
    });
    _renderer._component.addMouseListener(new MouseAdapter() {
      public void mouseReleased(MouseEvent event) {
        if (event.getButton() == MouseEvent.BUTTON1 || event.getButton() == MouseEvent.NOBUTTON) {
          Point p = _renderer._component.getLocationOnScreen();
          Point m = event.getLocationOnScreen();
          if ((m.x > p.x) && (m.y > p.y) && (m.y < p.y + _renderer._component.getHeight()) && (m.x < p.x + _renderer._component.getWidth())) {
            boolean wasSelected = _renderer._node.isSelected();
            WiredTreeSelectionModel selModel = _tree.getModel().getSelectionModel();
            if (event.isControlDown()) {
              if (wasSelected) {
                selModel.removeSelectionPath(_tree.getEditingPath());
              } else {
                selModel.addSelectionPath(_tree.getEditingPath());
              }
            } else {
              if (wasSelected && selModel.getSelectionCount() == 1) {
                  selModel.removeSelectionPath(_tree.getEditingPath());
              } else {
                selModel.setSelectionPath(_tree.getEditingPath());
              }
            }
            _renderer._component.invalidate();
          }
        }
        fireEditingStopped();
        tree.repaint();
      }
    });
}

// -- Implementing TreeCellEditor : --------------------------------------------

  public Object getCellEditorValue() {
    return _renderer._node;
  }

  public Component getTreeCellEditorComponent(JTree tree, Object value, boolean isSelected, boolean expanded, boolean leaf, int row) {
    _renderer.getTreeCellRendererComponent(tree, value, isSelected, expanded, leaf, row, true);
    return _renderer;
  }

  public boolean shouldSelectCell(EventObject anEvent) {
    return false;
  }

// -----------------------------------------------------------------------------
}
