// Copyright 2005, FreeHEP.
package hep.wired.variable;

import hep.wired.util.SwingEventMulticaster;

import javax.swing.event.ChangeListener;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: AbstractVariable.java 8598 2006-08-14 20:38:02Z duns $
 */

public abstract class AbstractVariable implements Variable {
    private String name;

    private Object object;

    private String unit;

    private String description;

    private boolean enabled;

    private ChangeListener listener;

    AbstractVariable(String name, Object object, String unit, String description) {
        this.name = name;
        this.object = object;
        this.unit = (unit != null) ? unit : "";
        this.description = (description != null) ? description : "";
        this.enabled = true;
    }

    public void addChangeListener(ChangeListener a) {
        listener = SwingEventMulticaster.add(listener, a);
    }

    public void removeChangeListener(ChangeListener r) {
        listener = SwingEventMulticaster.remove(listener, r);
    }

    public String getName() {
        return name;
    }

    public Object getObject() {
        return object;
    }

    public String getUnit() {
        return unit;
    }

    public String getDescription() {
        return description;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(boolean state) {
        if (enabled == state)
            return;
        enabled = state;

        fireStateChanged(VariableChangeEvent.STATE);
    }

    protected void fireStateChanged(int type) {
        if (listener != null) {
            listener.stateChanged(new VariableChangeEvent(this, type));
        }
    }
}
