// Copyright 2005, FreeHEP.
package hep.wired.variable;

import java.lang.reflect.Field;

import org.freehep.xml.io.XMLIOManager;
import org.jdom.DataConversionException;
import org.jdom.Element;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: BooleanVariable.java 8598 2006-08-14 20:38:02Z duns $
 */

public class BooleanVariable extends AbstractVariable {

    private boolean def;

    public BooleanVariable(String name, Object object, boolean def,
            String description) {
        super(name, object, null, description);
        this.def = def;
        setVariable(def);
    }

    /**
     * Returns the value of a named variable.
     */
    public boolean getBooleanVariable() {
        try {
            Field f = getObject().getClass().getDeclaredField(getName());
            f.setAccessible(true);
            if (f.getType() != boolean.class)
                throw new IllegalArgumentException("Variable with name '"
                        + getName() + "' not of type 'boolean'.");

            return f.getBoolean(getObject());
        } catch (NoSuchFieldException e) {
            throw new IllegalArgumentException("Variable with name '"
                    + getName() + "' does not exist as 'public' in class '"
                    + getObject().getClass() + "'.");
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException(
                    "No access to variable with name '" + getName() + "'.");
        }
    }

    public boolean setVariable(boolean value) {
        try {
            Field f = getObject().getClass().getDeclaredField(getName());
            f.setAccessible(true);
            if (f.getType() != boolean.class) {
                throw new IllegalArgumentException("Variable with name '"
                        + getName() + "' not of type 'boolean'.");
            }

            if (f.getBoolean(getObject()) == value)
                return false;

            f.setBoolean(getObject(), value);
        } catch (NoSuchFieldException e) {
            throw new IllegalArgumentException("Variable with name '"
                    + getName() + "' does not exist as 'public' in class '"
                    + getObject().getClass() + "'.");
        } catch (IllegalAccessException e) {
            throw new IllegalArgumentException(
                    "No access to variable with name '" + getName() + "'.");
        }
        fireStateChanged(VariableChangeEvent.VALUE);
        return true;
    }

    public Object getValue() {
        return new Boolean(getBooleanVariable());
    }

    public boolean setValue(Object value) {
        if (!(value instanceof Boolean)) {
            throw new IllegalArgumentException("Variable with name '"
                    + getName() + "' cannot handle value of class '"
                    + value.getClass() + "'.");
        }

        return setVariable(((Boolean) value).booleanValue());
    }

    public int compareTo(Object o) {
        if (o instanceof BooleanVariable) {
            BooleanVariable b = (BooleanVariable) o;
            boolean b1 = ((Boolean) getValue()).booleanValue();
            boolean b2 = ((Boolean) b.getValue()).booleanValue();
            return b1 == b2 ? 0 : b2 ? -1 : 1;
        }
        return 1;
    }

    public boolean getDefault() {
        return def;
    }

    public void saveValue(Element element) {
        element.setAttribute("val", Boolean.toString(getBooleanVariable()));
    }

    public void restoreValue(Element element) {
        try {
            setVariable(element.getAttribute("val").getBooleanValue());
        } catch (DataConversionException dce) {
            throw new IllegalArgumentException(getClass() + ": "
                    + dce.getMessage());
        }
    }

    public String toString() {
        return Boolean.toString(getBooleanVariable());
    }

    public void save(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {

        nodeEl.setAttribute("def", Boolean.toString(def));
    }

    public void restore(XMLIOManager xmlioManager, org.jdom.Element nodeEl) {
        try {
            def = nodeEl.getAttribute("def").getBooleanValue();
        } catch (DataConversionException dce) {
            throw new IllegalArgumentException(getClass() + ": "
                    + dce.getMessage());
        }
    }
}
