// Copyright 2005-2006, FreeHEP.
package hep.wired.variable;

import hep.wired.util.SwingEventMulticaster;
import hep.wired.util.slider.NumberSlider;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.Format;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.freehep.swing.layout.TableLayout;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: VariableSlider.java 8598 2006-08-14 20:38:02Z duns $
 */
public class VariableSlider extends JPanel implements ChangeListener {

    private VariableSliderModel model;
    
    private ChangeListener listener;

    protected JLabel sliderName;

    protected NumberSlider slider;

    protected JTextField sliderValue;

    protected JLabel sliderUnit;

    protected JButton sliderReset;

    public VariableSlider(VariableSliderModel newModel) {
        setLayout(new TableLayout());

        slider = new NumberSlider();
        sliderName = new JLabel("");
        sliderName.setHorizontalAlignment(JTextField.CENTER);
        sliderValue = new JTextField(10);
        sliderValue.setHorizontalAlignment(JTextField.RIGHT);
        sliderUnit = new JLabel("");
        sliderReset = new JButton("Reset");

        String insets = " [3 3 3 3] ";

        add(sliderName, "0 0" + insets + "w");
        add(slider, "0 1 2 1" + insets + "wh");
        add(sliderValue, "0 2" + insets + "w");
        add(sliderUnit, "1 2" + insets + "");
        add(sliderReset, "0 3 2 1" + insets + "w");
          
        sliderValue.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                if (model == null) return;
                try {
                    model.setValue(sliderValue.getText());
                } catch (NumberFormatException nfe) {
                    sliderValue.setText(getFormat().format(model.getNumber()));
                }
                sliderValue.selectAll();
            }
        });
        
        sliderReset.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (model != null) model.resetValue();
            } 
        });

        setModel(newModel);
    }

    public void setModel(VariableSliderModel newModel) {
        if (model != null) {
            model.removeChangeListener(this);
        }
        
        model = newModel;
        
        if (model != null) {
            model.addChangeListener(this);
            slider.setModel(model);
        }
        stateChanged(new ChangeEvent(this));
    }    
    
    public VariableSliderModel getModel() {
        return model;
    }
    
    public void setFormat(Format format) {
        slider.setFormat(format);
    }
    
    public Format getFormat() {
        return slider.getFormat();
    }
    
    public void addChangeListener(ChangeListener a) {
        listener = SwingEventMulticaster.add(listener, a);
    }

    public void removeChangeListener(ChangeListener r) {
        listener = SwingEventMulticaster.remove(listener, r);
    }
    
    private void fireStateChanged() {
        if (listener != null) {
            listener.stateChanged(new ChangeEvent(this));
        }
    }

    public void stateChanged(ChangeEvent e) {
        sliderName.setText(model != null ? model.getName() : "");
        sliderUnit.setText(model != null ? model.getUnit() : "");
        if (model != null) {
            sliderValue.setText(getFormat().format(model.getNumber()));
            sliderReset.setText("Default ("+getFormat().format(model.getDefaultNumber())+")");
        } else {
            sliderValue.setText("");
            sliderReset.setText("");
        }
        setEnabled(model != null ? model.isEnabled() : false);
        fireStateChanged();
    }
    
    public void setEnabled(boolean state) {
        super.setEnabled(state);
        sliderName.setEnabled(state);
        slider.setEnabled(state);
        sliderValue.setEnabled(state);
        sliderUnit.setEnabled(state);
        if ((state) && (model != null)) {
            sliderReset.setEnabled(!model.getDefaultNumber().equals(model.getNumber()));
        } else {
            sliderReset.setEnabled(false);
        }
    }

    // for testing
    private static int n;
    protected double test;
    
    public static void main(String[] args) {
        n = 1;

        DoubleVariableSliderModel m = new DoubleVariableSliderModel();
        m.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                System.err.println("Change: " + n + " srcClass: "
                        + e.getSource().getClass());
                n++;
            }
        });
        VariableSlider s = new VariableSlider(m);
        DoubleVariable d = new DoubleVariable("test", s, 0, 1000, 100, "kg",
                "Weight");
        m.setVariable(d);
        
        JFrame f = new JFrame();
        f.getContentPane().add(s);
        f.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        f.pack();
        f.setVisible(true);
    }
}
