// Copyright 2004-2005, FreeHEP.
package hep.wired.variable;

import hep.wired.util.SwingEventMulticaster;
import hep.wired.util.slider.BoundedNumberRangeModel;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: VariableSliderModel.java 8598 2006-08-14 20:38:02Z duns $
 */
public abstract class VariableSliderModel implements BoundedNumberRangeModel, ChangeListener {

    Variable variable;
    ChangeListener listener;
    
    public VariableSliderModel() {
       this(null);
    }

    public VariableSliderModel(Variable variable) {
        setVariable(variable);
    }

    public String getName() {
        return variable != null ? variable.getName() : "";
    }
    
    public boolean getValueIsAdjusting() {
        return false;
    }

    public void setValueIsAdjusting(boolean adjusting) {
        // ignored
    }

    public void setVariable(Variable newVariable) {
        if (variable != null) {
            variable.removeChangeListener(this);
        }

        variable = newVariable;
        if (variable != null) {
            variable.addChangeListener(this);
        }
        fireStateChanged();
    }

    public String getUnit() {
        return (variable != null) ? variable.getUnit() : "";
    }

    public boolean isEnabled() {
        return (variable != null) ? variable.isEnabled() : false;
    }
    
    public void addChangeListener(ChangeListener a) {
        listener = SwingEventMulticaster.add(listener, a);
    }
    
    public void removeChangeListener(ChangeListener r) {
        listener = SwingEventMulticaster.remove(listener, r);
    }
    
    public void fireStateChanged() {
        if (listener != null) {
            listener.stateChanged(new ChangeEvent(this));
        }
    }
    
    // forward the changes.
    public void stateChanged(ChangeEvent event) {
        fireStateChanged();
    }
    
    public abstract Number getNumber();
    
    public abstract Number getDefaultNumber();
    
    public abstract void resetValue();
    
    public abstract void setValue(String number) throws NumberFormatException;
}
