// Copyright 2005, FreeHEP.
package hep.wired.variable;

import hep.wired.util.SortableListTableModel;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * 
 * @author Mark Donszelmann
 * @version $Id: VariableTableModel.java 13931 2012-02-09 00:58:03Z onoprien $
 */
public class VariableTableModel extends SortableListTableModel implements ChangeListener {

  public static final int NAME = 0;
  public static final int VALUE = 1;
  public static final int UNIT = 2;
  private List/* <Variable> */ variableList;

  public VariableTableModel() {
    this(Collections.EMPTY_LIST);
  }

  public VariableTableModel(List /* <Variable> */ variables) {
    super(new String[]{"Name", "Value", "Unit"}, new Class[]{
              String.class, Class.class, String.class}, variables);
    setVariables(variables);
  }

  public void setVariables(List newVariableList) {
    if (variableList != null) {
      for (Iterator i = variableList.iterator(); i.hasNext();) {
        ((Variable) i.next()).removeChangeListener(this);
      }
    }

    variableList = newVariableList;
    for (Iterator i = variableList.iterator(); i.hasNext();) {
      ((Variable) i.next()).addChangeListener(this);
    }

    setKeys(variableList);
    // implicit structurechanged...
  }

  public int getRowCount() {
    return variableList.size();
  }

  public boolean isCellEditable(int row, int col) {
    // NOTE did not work, since just after editing started an update from
    // the slider comes in,
    // caused by the row selection, which cancels the editing.
    // We need to sort out how to prevent that update from cancelling the
    // editing.
    Variable variable = (Variable) variableList.get(row);
    return (col == VALUE) && (variable instanceof BooleanVariable); // (col
    // ==
    // VALUE);
  }

  public Object getValueAt(int row, int col) {
    Variable variable = (Variable) variableList.get(row);
    switch (col) {
      case NAME:
        return variable.getName();
      case VALUE:
        return variable.getValue();
      case UNIT:
        return variable.getUnit();
      default:
        return "";
    }
  }

  public void setValueAt(Object value, int row, int col) {
    Variable variable = (Variable) variableList.get(row);
    switch (col) {
      case VALUE:
        variable.setValue(value);
        fireTableCellUpdated(row, col);
        changed();
        return;
      case NAME:
      case UNIT:
      default:
        return;
    }
  }

  protected int compareKeys(Object key1, Object key2, int col,
          boolean ascending) {
    Variable v1 = (Variable) key1;
    Variable v2 = (Variable) key2;
    switch (col) {
      default:
      case NAME:
        return (ascending ? 1 : -1)
                * (v1.getName().compareTo(v2.getName()));
      case VALUE:
        return (ascending ? 1 : -1) * v1.compareTo(v2);

      case UNIT:
        return (ascending ? 1 : -1)
                * (v1.getUnit().compareTo(v2.getUnit()));
    }
  }

  protected void changed() {
  }

  // forward variable changes
  public void stateChanged(ChangeEvent event) {
    fireTableDataChanged();
  }
}
